def cons(x, y):
    '''
    cons(x, y) takes its two arguments and puts them into a python tuple.
    This tuple can be used to represent a node in a linked list, or a pair
    '''
    return (x, y)

def car(x):
    '''
    car(cons(x, y)) returns x. In other words, car returns the head of a linked list or the left item of a pair
    '''
    return x[0]

def cdr(x):
    '''
    cdr(cons(x, y)) returns y. In other words, cdr returns the tail of a linked list or the right item of a pair
    '''
    return x[1]

def caar(x):
    return car(car(x))

def cddr(x):
    return cdr(cdr(x))

def cdar(x):
    return cdr(car(x))

def cadr(x):
    return car(cdr(x))

def is_null(x):
    '''
    is_null(null) returns true
    is_null(()) returns true
    In all other cases is_null returns false
    '''
    return x == ()

def is_pair(x):
    '''
    is_pair(x) returns true if x is a pair/2-tuple/cons cell or null
    if x is anything else it returns false
    '''
    return is_null(x) or (isinstance(x, tuple) and len(x) == 2)

# null: Final = ()
null = ()

def ls(*argv):
    '''
    ls(*argv) takes a list of arguments and turns them into a lisp style list of cons cells
    so ls(1,2,3,4,5) => cons(1, cons(2, cons(3, cons(4, cons(5, ())))))
    '''
    if len(argv) is 0:
        return null
    else:
        return cons(argv[0], ls(*argv[1:]))


if __name__ == "__main__":
    print("Nothing in main at the moment")




from distutils.core import setup
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()


setup(
  name = 'pylispy',
  packages = ['pylispy'],   # Chose the same as "name"
  version = '0.5.0',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'Basic lisp functions in python',   # Give a short description about your library
  author = 'David Zabner',                   # Type in your name
  author_email = 'david.zabner@gmail.com',      # Type in your E-Mail
  long_description=long_description,
  long_description_content_type='text/markdown',
  url = 'https://github.com/nw0428/pylisp',   # Provide either the link to your github or to your website
  download_url = 'https://github.com/nw0428/pylisp/archive/v_050.tar.gz',    # I explain this later on
  keywords = ['Lisp', 'Lists', 'LISP', 'car', 'cdr', 'cons'],   # Keywords that define your package best
  classifiers=[
    'Development Status :: 4 - Beta',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Education',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
  ],
)
